from __future__ import print_function

import unittest

from svgelements import *


class TestElementElement(unittest.TestCase):
    """These tests ensure the performance of the SVGElement basecase."""

    def test_element_id(self):
        values = {'id': 'my_id', 'random': True}
        r = Rect(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = Circle(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = Ellipse(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = Polygon(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = Polyline(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = SimpleLine(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = Path(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = SVGImage(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])
        r = SVGText(values)
        self.assertEqual(values['id'], r.values['id'])
        self.assertEqual(values['random'], r.values['random'])
        self.assertRaises(KeyError, lambda: r.values['not_there'])

    def test_element_merge(self):
        values = {'id': 'my_id', 'random': True}
        r = Rect(values, random=False, tat='awesome')
        self.assertEqual(r.values['id'], values['id'])
        self.assertNotEqual(r.values['random'], values['random'])
        self.assertEqual(r.values['tat'], 'awesome')

        r = Rect(fill='red')
        self.assertEqual(r.fill, '#f00')

    def test_element_propagate(self):

        values = {'id': 'my_id', 'random': True}
        r = Rect(values, random=False, tat='awesome')
        r = Rect(r)
        self.assertEqual(r.values['id'], values['id'])
        self.assertNotEqual(r.values['random'], values['random'])
        self.assertEqual(r.values['tat'], 'awesome')

        r = Rect(fill='red')
        r = Rect(r)
        self.assertEqual(r.fill, '#f00')
        r = Rect(stroke='red')
        r = Rect(r)
        self.assertEqual(r.stroke, '#f00')

        r = Rect(width=20)
        r = Rect(r)
        self.assertEqual(r.width, 20)

        p = Path('M0,0 20,0 0,20z M20,20 40,20 20,40z', fill='red')
        p2 = Path(p.subpath(1))
        p2[0].start = None
        self.assertEqual(p2, 'M20,20 40,20 20,40z')
        self.assertEqual(p2.fill, 'red')




from __future__ import print_function

import unittest

from svgelements import *


class TestPathMatrix(unittest.TestCase):
    """Tests the functionality of the Matrix element."""

    def test_rotate_css_angles(self):
        matrix = Matrix("rotate(90, 100,100)")
        path = Path("M0,0Z")
        path *= matrix
        self.assertEqual("M 200,0 Z", path.d())
        matrix = Matrix("rotate(90deg, 100,100)")
        path = Path("M0,0Z")
        path *= matrix
        self.assertEqual("M 200,0 Z", path.d())
        matrix = Matrix("rotate(0.25turn, 100,100)")
        path = Path("M0,0Z")
        path *= matrix
        self.assertEqual("M 200,0 Z", path.d())
        matrix = Matrix("rotate(100grad, 100,100)")
        path = Path("M0,0Z")
        path *= matrix
        self.assertEqual("M 200,0 Z", path.d())
        matrix = Matrix("rotate(1.5707963267948966rad, 100,100)")
        path = Path("M0,0Z")
        path *= matrix
        self.assertEqual("M 200,0 Z", path.d())

    def test_matrix_multiplication(self):
        self.assertEqual(Matrix("scale(0.2) translate(-5,-5)"), Matrix("translate(-5,-5)") * Matrix("scale(0.2)"))
        self.assertEqual(Matrix("translate(-5,-5) scale(0.2)"), Matrix("scale(0.2)") * Matrix("translate(-5,-5)"))

    def test_rotate_css_distance(self):
        matrix = Matrix("rotate(90deg,100cm,100cm)")
        matrix.render(ppi=DEFAULT_PPI)
        path = Path("M0,0z")
        path *= matrix
        d = Length("1cm").value(ppi=DEFAULT_PPI)
        p2 = Path("M 200,0 Z") * Matrix("scale(%f)" % d)
        p2.values[SVG_ATTR_VECTOR_EFFECT] = SVG_VALUE_NON_SCALING_STROKE
        self.assertEqual(p2, path)

    def test_skew_single_value(self):
        m0 = Matrix("skew(15deg,0deg)")
        m1 = Matrix("skewX(15deg)")
        self.assertEqual(m0, m1)
        m0 = Matrix("skew(0deg,15deg)")
        m1 = Matrix("skewY(15deg)")
        self.assertEqual(m0, m1)

    def test_scale_single_value(self):
        m0 = Matrix("scale(2,1)")
        m1 = Matrix("scaleX(2)")
        self.assertEqual(m0, m1)
        m0 = Matrix("scale(1,2)")
        m1 = Matrix("scaleY(2)")
        self.assertEqual(m0, m1)

    def test_translate_single_value(self):
        m0 = Matrix("translate(500cm,0)")
        m1 = Matrix("translateX(500cm)")
        self.assertEqual(m0, m1)
        m0 = Matrix("translate(0,500cm)")
        m1 = Matrix("translateY(500cm)")
        self.assertEqual(m0, m1)
        m0 = Matrix("translate(500cm)")
        m1 = Matrix("translateX(500cm)")
        self.assertEqual(m0, m1)

    def test_translate_css_value(self):
        m0 = Matrix("translate(50mm,5cm)")
        m1 = Matrix("translate(5cm,5cm)")
        self.assertEqual(m0, m1)

    def test_rotate_css_value(self):
        m0 = Matrix("rotate(90deg, 50cm,50cm)", ppi=DEFAULT_PPI)
        m1 = Matrix("rotate(0.25turn, 500mm,500mm)", ppi=DEFAULT_PPI)
        self.assertEqual(m0, m1)

    def test_transform_translate(self):
        matrix = Matrix("translate(5,4)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, "z").closed()
        path *= matrix
        self.assertEqual("M 5,4 L 5,104 L 105,104 L 105,4 L 5,4 Z", path.d())

    def test_transform_scale(self):
        matrix = Matrix("scale(2)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, 'z').closed()
        path *= matrix
        self.assertEqual("M 0,0 L 0,200 L 200,200 L 200,0 L 0,0 Z", path.d())

    def test_transform_rotate(self):
        matrix = Matrix("rotate(360)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, "z")
        path *= matrix
        self.assertAlmostEqual(path[0][1].x, 0)
        self.assertAlmostEqual(path[0][1].y, 0)

        self.assertAlmostEqual(path[1][1].x, 0)
        self.assertAlmostEqual(path[1][1].y, 100)

        self.assertAlmostEqual(path[2][1].x, 100)
        self.assertAlmostEqual(path[2][1].y, 100)
        self.assertAlmostEqual(path[3][1].x, 100)
        self.assertAlmostEqual(path[3][1].y, 0)
        self.assertAlmostEqual(path[4][1].x, 0)
        self.assertAlmostEqual(path[4][1].y, 0)

    def test_transform_value(self):
        matrix = Matrix("rotate(360,50,50)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, "z")
        path *= matrix
        self.assertAlmostEqual(path[0][1].x, 0)
        self.assertAlmostEqual(path[0][1].y, 0)

        self.assertAlmostEqual(path[1][1].x, 0)
        self.assertAlmostEqual(path[1][1].y, 100)

        self.assertAlmostEqual(path[2][1].x, 100)
        self.assertAlmostEqual(path[2][1].y, 100)
        self.assertAlmostEqual(path[3][1].x, 100)
        self.assertAlmostEqual(path[3][1].y, 0)
        self.assertAlmostEqual(path[4][1].x, 0)
        self.assertAlmostEqual(path[4][1].y, 0)

    def test_transform_skewx(self):
        matrix = Matrix("skewX(10,50,50)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, "z").closed()
        path *= matrix
        self.assertEqual(
            "M -8.81634903542,0 L 8.81634903542,100 L 108.816349035,100 L 91.1836509646,0 L -8.81634903542,0 Z",
            path.d())

    def test_transform_skewy(self):
        matrix = Matrix("skewY(10, 50,50)")
        path = Path()
        path.move((0, 0), (0, 100), (100, 100), 100 + 0j, "z").closed()
        path *= matrix
        self.assertEqual(
            "M 0,-8.81634903542 L 0,91.1836509646 L 100,108.816349035 L 100,8.81634903542 L 0,-8.81634903542 Z",
            path.d())

    def test_matrix_repr_rotate(self):
        """
        [a c e]
        [b d f]
        """
        self.assertEqual(Matrix(0, 1, -1, 0, 0, 0), Matrix.rotate(radians(90)))

    def test_matrix_repr_scale(self):
        """
        [a c e]
        [b d f]
        """
        self.assertEqual(Matrix(2, 0, 0, 2, 0, 0), Matrix.scale(2))

    def test_matrix_repr_hflip(self):
        """
        [a c e]
        [b d f]
        """
        self.assertEqual(Matrix(-1, 0, 0, 1, 0, 0), Matrix.scale(-1, 1))

    def test_matrix_repr_vflip(self):
        """
        [a c e]
        [b d f]
        """
        self.assertEqual(Matrix(1, 0, 0, -1, 0, 0), Matrix.scale(1, -1))

    def test_matrix_repr_post_cat(self):
        """
        [a c e]
        [b d f]
        """
        m = Matrix.scale(2)
        m.post_cat(Matrix.translate(-20, -20))
        self.assertEqual(Matrix(2, 0, 0, 2, -20, -20), m)

    def test_matrix_repr_pre_cat(self):
        """
        [a c e]
        [b d f]
        """
        m = Matrix.translate(-20, -20)
        m.pre_cat(Matrix.scale(2))
        self.assertEqual(Matrix(2, 0, 0, 2, -20, -20), m)

    def test_matrix_point_rotated_by_matrix(self):
        matrix = Matrix()
        matrix.post_rotate(radians(90), 100, 100)
        p = matrix.point_in_matrix_space((50, 50))
        self.assertAlmostEqual(p[0], 150)
        self.assertAlmostEqual(p[1], 50)

    def test_matrix_point_scaled_in_space(self):
        matrix = Matrix()
        matrix.post_scale(2, 2, 50, 50)

        p = matrix.point_in_matrix_space((50, 50))
        self.assertAlmostEqual(p[0], 50)
        self.assertAlmostEqual(p[1], 50)

        p = matrix.point_in_matrix_space((25, 25))
        self.assertAlmostEqual(p[0], 0)
        self.assertAlmostEqual(p[1], 0)

        matrix.post_rotate(radians(45), 50, 50)
        p = matrix.point_in_matrix_space((25, 25))
        self.assertAlmostEqual(p[0], 50)

        matrix = Matrix()
        matrix.post_scale(0.5, 0.5)
        p = matrix.point_in_matrix_space((100, 100))
        self.assertAlmostEqual(p[0], 50)
        self.assertAlmostEqual(p[1], 50)

        matrix = Matrix()
        matrix.post_scale(2, 2, 100, 100)
        p = matrix.point_in_matrix_space((50, 50))
        self.assertAlmostEqual(p[0], 0)
        self.assertAlmostEqual(p[1], 0)

    def test_matrix_cat_identity(self):
        identity = Matrix()
        from random import random
        for i in range(50):
            q = Matrix(random(), random(), random(), random(), random(), random())
            p = copy(q)
            q.post_cat(identity)
            self.assertEqual(q, p)

    def test_matrix_pre_and_post_1(self):
        from random import random
        for i in range(50):
            tx = random() * 1000 - 500
            ty = random() * 1000 - 500
            rx = random() * 2
            ry = random() * 2
            a = random() * tau
            q = Matrix()
            q.post_translate(tx, ty)
            p = Matrix()
            p.pre_translate(tx, ty)
            self.assertEqual(p, q)

            q = Matrix()
            q.post_scale(rx, ry, tx, ty)
            p = Matrix()
            p.pre_scale(rx, ry, tx, ty)
            self.assertEqual(p, q)

            q = Matrix()
            q.post_rotate(a, tx, ty)
            p = Matrix()
            p.pre_rotate(a, tx, ty)
            self.assertEqual(p, q)

            q = Matrix()
            q.post_skew_x(a, tx, ty)
            p = Matrix()
            p.pre_skew_x(a, tx, ty)
            self.assertEqual(p, q)

            q = Matrix()
            q.post_skew_y(a, tx, ty)
            p = Matrix()
            p.pre_skew_y(a, tx, ty)
            self.assertEqual(p, q)

    def test_matrix_eval_repr(self):
        self.assertTrue(Matrix("rotate(20)") == eval(repr(Matrix("rotate(20)"))))
        self.assertFalse(Matrix("rotate(20)") != eval(repr(Matrix("rotate(20)"))))

    def test_matrix_reverse_scale(self):
        m1 = Matrix("scale(2)")
        m1.inverse()
        m2 = Matrix("scale(0.5)")
        self.assertEqual(m1, m2)
        m1.inverse()
        self.assertEqual(m1, "scale(2)")

    def test_matrix_reverse_translate(self):
        m1 = Matrix("translate(20,20)")
        m1.inverse()
        self.assertEqual(m1, Matrix("translate(-20,-20)"))

    def test_matrix_reverse_rotate(self):
        m1 = Matrix("rotate(30)")
        m1.inverse()
        self.assertEqual(m1, Matrix("rotate(-30)"))

    def test_matrix_reverse_skew(self):
        m1 = Matrix("skewX(1)")
        m1.inverse()
        self.assertEqual(m1, Matrix("skewX(-1)"))

        m1 = Matrix("skewY(1)")
        m1.inverse()
        self.assertEqual(m1, Matrix("skewY(-1)"))

    def test_matrix_reverse_scale_translate(self):
        m1 = Matrix("scale(2) translate(40,40)")
        m1.inverse()
        self.assertEqual(m1, Matrix("translate(-40,-40) scale(0.5)"))



from __future__ import print_function

import unittest

from svgelements import *


class TestElementAngle(unittest.TestCase):
    """These tests ensure the basic functions of the Angle element."""

    def test_angle_init(self):
        self.assertEqual(Angle.degrees(90).as_turns, 0.25)
        self.assertEqual(Angle.degrees(180).as_turns, 0.50)
        self.assertEqual(Angle.degrees(360).as_turns, 1.0)
        self.assertEqual(Angle.degrees(720).as_turns, 2.0)
        self.assertEqual(Angle.radians(tau).as_turns, 1.0)
        self.assertEqual(Angle.radians(tau / 50.0).as_turns, 1.0 / 50.0)
        self.assertEqual(Angle.gradians(100).as_turns, 0.25)
        self.assertEqual(Angle.turns(100).as_turns, 100)
        self.assertEqual(Angle.gradians(100).as_gradians, 100)
        self.assertEqual(Angle.degrees(100).as_degrees, 100)
        self.assertEqual(Angle.radians(100).as_radians, 100)
        self.assertEqual(Angle.parse("90deg").as_radians, tau / 4.0)
        self.assertEqual(Angle.parse("90turn").as_radians, tau * 90)

    def test_angle_equal(self):
        self.assertEqual(Angle.degrees(0), Angle.degrees(-360))
        self.assertEqual(Angle.degrees(0), Angle.degrees(360))
        self.assertEqual(Angle.degrees(0), Angle.degrees(1080))
        self.assertNotEqual(Angle.degrees(0), Angle.degrees(180))
        self.assertEqual(Angle.degrees(0), Angle.turns(5))

    def test_orth(self):
        self.assertTrue(Angle.degrees(0).is_orthogonal())
        self.assertTrue(Angle.degrees(90).is_orthogonal())
        self.assertTrue(Angle.degrees(180).is_orthogonal())
        self.assertTrue(Angle.degrees(270).is_orthogonal())
        self.assertTrue(Angle.degrees(360).is_orthogonal())

        self.assertFalse(Angle.degrees(1).is_orthogonal())
        self.assertFalse(Angle.degrees(91).is_orthogonal())
        self.assertFalse(Angle.degrees(181).is_orthogonal())
        self.assertFalse(Angle.degrees(271).is_orthogonal())
        self.assertFalse(Angle.degrees(361).is_orthogonal())



from __future__ import print_function

import unittest

from svgelements import *


class TestPath(unittest.TestCase):
    """Tests of the SVG Path element."""

    def test_subpaths(self):
        path = Path("M0,0 50,50 100,100Z M0,100 50,50, 100,0")
        for i, p in enumerate(path.as_subpaths()):
            if i == 0:
                self.assertEqual(p.d(), "M 0,0 L 50,50 L 100,100 Z")
            elif i == 1:
                self.assertEqual(p.d(), "M 0,100 L 50,50 L 100,0")
            self.assertLessEqual(i, 1)

    def test_move_quad_smooth(self):
        path = Path()
        path.move((4, 4), (20, 20), (25, 25), 6 + 3j)
        path.quad((20, 33), (100, 100))
        path.smooth_quad((13, 45), (16, 16), (34, 56), "z").closed()
        self.assertEqual(path.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T 13,45 T 16,16 T 34,56 T 4,4 Z")

    def test_move_cubic_smooth(self):
        path = Path()
        path.move((4, 4), (20, 20), (25, 25), 6 + 3j)
        path.cubic((20, 33), (25, 25), (100, 100))
        path.smooth_cubic((13, 45), (16, 16), (34, 56), "z").closed()
        self.assertEqual(path.d(), "M 4,4 L 20,20 L 25,25 L 6,3 C 20,33 25,25 100,100 S 13,45 16,16 S 34,56 4,4 Z")

    def test_convex_hull(self):
        pts = (3, 4), (4, 6), (18, -2), (9, 0)
        hull = [e for e in Point.convex_hull(pts)]
        self.assertEqual([(3, 4), (9, 0), (18, -2), (4, 6)], hull)

        # bounding box and a bunch of random numbers that must be inside.
        pts = [(100, 100), (100, -100), (-100, -100), (-100, 100)]
        from random import randint
        for i in range(50):
            pts.append((randint(-99, 99), randint(-99, 99)))
        hull = [e for e in Point.convex_hull(pts)]
        for p in hull:
            self.assertEqual(abs(p[0]), 100)
            self.assertEqual(abs(p[1]), 100)

    def test_reverse_path_q(self):
        path = Path("M1,0 22,7 Q 17,17 91,2")
        path.reverse()
        self.assertEqual(path, Path("M 91,2 Q 17,17 22,7 L 1,0"))

    def test_reverse_path_multi_move(self):
        path = Path("M1,0 M2,0 M3,0")
        path.reverse()
        self.assertEqual(path, "M3,0 M2,0 M1,0")
        path = Path("M1,0z M2,0z M3,0z")
        path.reverse()
        self.assertEqual(path, "M3,0 Z M2,0 Z M1,0 Z")

    def test_reverse_path_multipath(self):
        path = Path("M1,0 22,7 Q 17,17 91,2M0,0zM20,20z")
        path.reverse()
        self.assertEqual(path, Path("M20,20zM0,0zM 91,2 Q 17,17 22,7 L 1,0"))

    def test_path_mult_sideeffect(self):
        path = Path("M1,1 10,10 Q 17,17 91,2 T 9,9 C 40,40 20,0, 9,9 S 60,50 0,0 A 25,25 -30 0,1 30,30 z")
        q = path * "scale(2)"
        self.assertEqual(path, "M1,1 10,10 Q 17,17 91,2 T 9,9 C 40,40 20,0, 9,9 S 60,50 0,0 A 25,25 -30 0,1 30,30 z")

    def test_subpath_imult_sideeffect(self):
        path = Path("M1,1 10,10 Q 17,17 91,2 T 9,9 C 40,40 20,0, 9,9 S 60,50 0,0 A 25,25 -30 0,1 30,30 zM50,50z")
        self.assertEqual(
            path,
            "M1,1 10,10 Q 17,17 91,2 T 9,9 C 40,40 20,0, 9,9 S 60,50 0,0 A 25,25 -30 0,1 30,30 zM50,50z")
        for p in path.as_subpaths():
            p *= "scale(2)"
        self.assertEqual(
            path,
            "M 2,2 L 20,20 Q 34,34 182,4 T 18,18 C 80,80 40,0 18,18 S 120,100 0,0 A 50,50 -30 0,1 60,60 ZM100,100z")

    def test_subpath_reverse(self):
        #Issue 45
        p = Path("M0,0 1,1")
        p.reverse()
        self.assertEqual(p, "M1,1 0,0")

        p = Path("M0,0 M1,1")
        p.reverse()
        self.assertEqual(p, "M1,1 M0,0")

        p = Path("M1,1 L5,5M2,1 L6,5M3,1 L7,5")
        subpaths = list(p.as_subpaths())
        subpaths[1].reverse()
        self.assertEqual("M 1,1 L 5,5 M 6,5 L 2,1 M 3,1 L 7,5", str(p))
        subpaths[1].reverse()
        self.assertEqual("M 1,1 L 5,5 M 2,1 L 6,5 M 3,1 L 7,5", str(p))

        p = Path("M1,1 L5,5M2,1 L6,5ZM3,1 L7,5")
        subpaths = list(p.as_subpaths())
        subpaths[1].reverse()
        self.assertEqual("M 6,5 L 2,1 Z", str(subpaths[1]))
        self.assertEqual("M 1,1 L 5,5 M 6,5 L 2,1 Z M 3,1 L 7,5", str(p))

        p = Path("M1,1 L5,5M2,1 6,5 100,100 200,200 ZM3,1 L7,5")
        subpaths = list(p.as_subpaths())
        subpaths[1].reverse()
        self.assertEqual("M 1,1 L 5,5 M 200,200 L 100,100 L 6,5 L 2,1 Z M 3,1 L 7,5", str(p))

    def test_validation_delete(self):
        p = Path("M1,1 M2,2 M3,3 M4,4")
        del p[2]
        self.assertEqual(p, "M1,1 M2,2 M4,4")
        p = Path("M0,0 L 1,1 L 2,2 L 3,3 L 4,4z")
        del p[3]
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 4,4z")
        p = Path("M0,0 L 1,1 L 2,2 M 3,3 L 4,4z")
        del p[3]
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 4,4z")

    def test_validation_insert(self):
        p = Path("M1,1 M2,2 M4,4")
        p.insert(2, "M3,3")
        self.assertEqual(p, "M1,1 M2,2 M3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2 L 4,4")
        p.insert(3, "L3,3")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4")

    def test_validation_append(self):
        p = Path("M1,1 M2,2 M3,3")
        p.append("M4,4")
        self.assertEqual(p, "M1,1 M2,2 M3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2 L 3,3")
        p.append("L4,4")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4")
        p.append("Z")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4 Z")

        p = Path("M1,1 M2,2")
        p.append("M3,3 M4,4")
        self.assertEqual(p, "M1,1 M2,2 M3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2")
        p.append("L 3,3 L4,4")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4")
        p.append("Z")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4 Z")

    def test_validation_extend(self):
        p = Path("M1,1 M2,2")
        p.extend(Path("M3,3 M4,4"))
        self.assertEqual(p, "M1,1 M2,2 M3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2")
        p.extend(Path("L 3,3 L4,4"))
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4")
        p.extend(Path("Z"))
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4 Z")

        p = Path("M1,1 M2,2")
        p.extend("M3,3 M4,4")
        self.assertEqual(p, "M1,1 M2,2 M3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2")
        p.extend("L 3,3 L4,4")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4")
        p.extend("Z")
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 L 3,3 L 4,4 Z")

    def test_validation_setitem(self):
        p = Path("M1,1 M2,2 M3,3 M4,4")
        p[2] = Line(None, (3,3))
        self.assertEqual(p, "M1,1 M2,2 L3,3 M4,4")
        p = Path("M0,0 L 1,1 L 2,2 L 3,3 L 4,4z")
        p[3] = Move(None, (3,3))
        self.assertEqual(p, "M0,0 L 1,1 L 2,2 M3,3 L 4,4z")

    def test_validation_setitem_str(self):
        p = Path("M1,1 M2,2 M3,3 M4,4")
        p[2] = "L3,3"
        self.assertEqual(p, Path("M1,1 M2,2 L3,3 M4,4"))
        p = Path("M0,0 L 1,1 L 2,2 L 3,3 L 4,4z")
        p[3] = "M3,3"
        self.assertEqual(p, Path("M0,0 L 1,1 L 2,2 M3,3 L 4,4z"))

    def test_arc_start_t(self):
        m = Path("m 0,0 a 5.01,5.01 180 0,0 0,10 z"
                 "m 0,0 a 65,65 180 0,0 65,66 z")
        for a in m:
            if isinstance(a, Arc):
                start_t = a.get_start_t()
                a_start = a.point_at_t(start_t)
                self.assertEqual(a.start, a_start)
                self.assertEqual(a.end, a.point_at_t(a.get_end_t()))

    def test_relative_roundabout(self):
        m = Path("m 0,0 a 5.01,5.01 180 0,0 0,10 z")
        self.assertEqual(m.d(), "m 0,0 a 5.01,5.01 180 0,0 0,10 z")
        m = Path("M0,0 1,1 z")
        self.assertEqual(m.d(), "M 0,0 L 1,1 z")
        self.assertEqual(m.d(relative=True), "m 0,0 l 1,1 z")
        self.assertEqual(m.d(relative=False), "M 0,0 L 1,1 Z")
        m = Path("M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 Q 180,167 13,45 T 16,16 T 34,56 T 4,4 z")
        self.assertEqual(m.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 Q 180,167 13,45 T 16,16 T 34,56 T 4,4 z")
        self.assertEqual(m.d(smooth=False), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 Q 180,167 13,45 Q -154,-77 16,16 Q 186,109 34,56 Q -118,3 4,4 z")
        self.assertEqual(m.d(smooth=True), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T 13,45 T 16,16 T 34,56 T 4,4 z")
        self.assertEqual(m.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 Q 180,167 13,45 T 16,16 T 34,56 T 4,4 z")

    def test_path_z_termination(self):
        m = Path("M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 Z")
        self.assertEqual(m.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 4,4 Z")
        m = Path("M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T Z")
        self.assertEqual(m.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T 4,4 Z")
        m = Path("M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T z")
        self.assertEqual(m.d(), "M 4,4 L 20,20 L 25,25 L 6,3 Q 20,33 100,100 T 4,4 z")
        m = Path("m 0,0 1,1 A 5.01,5.01 180 0,0 z")
        self.assertEqual(m.d(), "m 0,0 l 1,1 A 5.01,5.01 180 0,0 0,0 z")
        m = Path("m0,0z")
        self.assertEqual(m.d(), "m 0,0 z")
        m = Path("M0,0Lz")
        self.assertEqual(m.d(), "M 0,0 L 0,0 z")

    def test_path_setitem_slice(self):
        m = Path("M0,0 1,1 z")
        m[1:] = 'L2,2z'
        self.assertEqual(m.d(), "M 0,0 L 2,2 z")
        self.assertTrue(m._is_valid())
        del m[1]
        self.assertEqual(m.d(), "M 0,0 z")
        self.assertTrue(m._is_valid())

        m = Path("M0,0z")
        m[:] = 'M1,1z'
        self.assertEqual(m.d(), "M 1,1 z")
        self.assertTrue(m._is_valid())

        m = Path("M0,0z")
        del m[:]
        self.assertEqual(m, '')
        self.assertTrue(m._is_valid())

        m = Path("M0,0z")
        m[0] = "M1,1"
        self.assertEqual(m.d(), "M 1,1 z")
        self.assertTrue(m._is_valid())
        m[1] = "z"
        self.assertTrue(m._is_valid())

        m = Path("M0,0z")
        del m[1]
        self.assertEqual(m.d(), "M 0,0")
        self.assertTrue(m._is_valid())

        m = Path("M0,0 1,1 z")
        m[3:] = "M5,5z"
        self.assertEqual(m.d(), "M 0,0 L 1,1 z M 5,5 z")
        self.assertTrue(m._is_valid())

        m = Path("M0,0 1,1 z")
        m[-1:] = "M5,5z"
        self.assertEqual(m.d(), "M 0,0 L 1,1 M 5,5 z")
        self.assertTrue(m._is_valid())

        m = Path("M0,0 1,1 z")
        def m_assign():
            m[-1] = 'M5,5z'
        self.assertRaises(ValueError, m_assign)



from __future__ import print_function

import io
import unittest

from svgelements import *


class TestDescriptiveElements(unittest.TestCase):

    def test_descriptive_element(self):
        q = io.StringIO('<?xml version="1.0" encoding="utf-8" ?>\n'
                        '<svg>'
                        '<title>Who?</title>'
                        '<desc>My Friend.</desc>'
                        '</svg>')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertEqual(len(q), 3)
        self.assertEqual(q[1].title, "Who?")
        self.assertEqual(q[2].desc, "My Friend.")




from __future__ import print_function

import unittest
from random import random

from svgelements import *


class TestElementPoint(unittest.TestCase):

    def test_point_init_string(self):
        p = Point("(0,24)")
        self.assertEqual(p, (0, 24))
        self.assertEqual(p, 0 + 24j)
        self.assertEqual(p, [0, 24])
        self.assertEqual(p, "(0,24)")

    def test_polar_angle(self):
        for i in range(1000):
            p = Point(random() * 50, random() * 50)
            a = random() * tau - tau / 2
            r = random() * 50
            m = Point.polar(p, a, r)
            self.assertAlmostEqual(Point.angle(p, m), a)

    def test_not_equal_unparsed(self):
        self.assertNotEqual(Point(0, 0), "string that doesn't parse to point")

    def test_dunder_iadd(self):
        p = Point(0)
        p += (1, 0)
        self.assertEqual(p, (1, 0))
        p += Point(1, 1)
        self.assertEqual(p, (2, 1))
        p += 1 + 2j
        self.assertEqual(p, (3, 3))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p += c()
        self.assertEqual(p, (4, 4))
        p += Point("-4,-4")
        self.assertEqual(p, (0, 0))
        p += 1
        self.assertEqual(p, (1, 0))
        self.assertRaises(TypeError, 'p += "hello"')

    def test_dunder_isub(self):
        p = Point(0)
        p -= (1, 0)
        self.assertEqual(p, (-1, 0))
        p -= Point(1, 1)
        self.assertEqual(p, (-2, -1))
        p -= 1 + 2j
        self.assertEqual(p, (-3, -3))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p -= c()
        self.assertEqual(p, (-4, -4))
        p -= Point("-4,-4")
        self.assertEqual(p, (0, 0))
        p -= 1
        self.assertEqual(p, (-1, 0))
        r = p - 1
        self.assertEqual(r, (-2, 0))
        self.assertRaises(TypeError, 'p -= "hello"')

    def test_dunder_add(self):
        p = Point(0)
        p = p + (1, 0)
        self.assertEqual(p, (1, 0))
        p = p + Point(1, 1)
        self.assertEqual(p, (2, 1))
        p = p + 1 + 2j
        self.assertEqual(p, (3, 3))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p = p + c()
        self.assertEqual(p, (4, 4))
        p = p + Point("-4,-4")
        self.assertEqual(p, (0, 0))
        p = p + 1
        self.assertEqual(p, (1, 0))
        self.assertRaises(TypeError, 'p = p + "hello"')

    def test_dunder_sub(self):
        p = Point(0)
        p = p - (1, 0)
        self.assertEqual(p, (-1, 0))
        p = p - Point(1, 1)
        self.assertEqual(p, (-2, -1))
        p = p - (1 + 2j)
        self.assertEqual(p, (-3, -3))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p = p - c()
        self.assertEqual(p, (-4, -4))
        p = p - Point("-4,-4")
        self.assertEqual(p, (0, 0))
        p = p - 1
        self.assertEqual(p, (-1, 0))
        self.assertRaises(TypeError, 'p = p - "hello"')

    def test_dunder_rsub(self):
        p = Point(0)
        p = (1, 0) - p
        self.assertEqual(p, (1, 0))
        p = Point(1, 1) - p
        self.assertEqual(p, (0, 1))
        p = (1 + 2j) - p
        self.assertEqual(p, (1, 1))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p = c() - p
        self.assertEqual(p, (0, 0))
        p = Point("-4,-4") - p
        self.assertEqual(p, (-4, -4))
        p = 1 - p
        self.assertEqual(p, (5, 4))
        self.assertRaises(TypeError, 'p = "hello" - p')

    def test_dunder_mult(self):
        """
        For backwards compatibility multiplication of points works like multiplication of complex variables.

        :return:
        """
        p = Point(2, 2)
        p *= (1, 0)
        self.assertEqual(p, (2, 2))
        p *= Point(1, 1)
        self.assertEqual(p, (0, 4))
        p *= 1 + 2j
        self.assertEqual(p, (-8, 4))

        class c:
            def __init__(self):
                self.x = 1
                self.y = 1

        p *= c()
        self.assertEqual(p, (-12, -4))
        p *= Point("-4,-4")
        self.assertEqual(p, (32, 64))
        p *= 1
        self.assertEqual(p, (32, 64))
        r = p * 1
        self.assertEqual(r, (32, 64))
        r *= "scale(0.1)"
        self.assertEqual(r, (3.2, 6.4))

    def test_dunder_transform(self):
        p = Point(4, 4)
        m = Matrix("scale(4)")
        p.matrix_transform(m)
        self.assertEqual(p, (16, 16))

    def test_move_towards(self):
        p = Point(4, 4)
        p.move_towards((6, 6), 0.5)
        self.assertEqual(p, (5, 5))

    def test_distance_to(self):
        p = Point(4, 4)
        m = p.distance_to((6, 6))
        self.assertEqual(m, 2 * sqrt(2))
        m = p.distance_to(4)
        self.assertEqual(m, 4)

    def test_angle_to(self):
        p = Point(0)
        a = p.angle_to((3, 3))
        self.assertEqual(a, Angle.parse("45deg"))
        a = p.angle_to((0, 3))
        self.assertEqual(a, Angle.parse("0.25turn"))
        a = p.angle_to((-3, 0))
        self.assertEqual(a, Angle.parse("200grad"))

    def test_polar(self):
        p = Point(0)
        q = p.polar_to(Angle.parse("45deg"), 10)
        self.assertEqual(q, (sqrt(2)/2 * 10, sqrt(2)/2 * 10))

    def test_reflected_across(self):
        p = Point(0)
        r = p.reflected_across((10,10))
        self.assertEqual(r, (20,20))